@@ -2,18 +2,18 @@
 
 ## ralf.herold@gmx.net - 2015-08-15
 ## part of https://github.com/rfhb/ctrdata
-# last edited: 2017-01-12
 #
-# note line endings are to be kep by using in
+# note line endings are to be kept by using in
 # .gitattributes for compatibility with cygwin:
 # *.sh  text eol=lf
 # *.php text eol=lf
 #
 # time euctr2json.sh:
-# 2015-08-15: 2.4 s for 221 documents: ~ 11 ms per trial (MacBookPro2011)
-# 2016-04-20: 1.2 s for 151 documents: ~  8 ms per trial (MacBookPro2011)
-# 2016-09-11: 1.2 s for 151 documents: ~  8 ms per trial (MacBookPro2015)
-# 2017-01-12: real 1m23.021s for 10978 documents ~ 8 ms per trial (MacBookPro2015)
+# 2015-08-15: real 2.4 s for 221 documents: ~ 11 ms per trial (MacBookPro2011)
+# 2016-04-20: real 1.2 s for 151 documents: ~  8 ms per trial (MacBookPro2011)
+# 2016-09-11: real 1.2 s for 151 documents: ~  8 ms per trial (MacBookPro2015)
+# 2017-01-12: real 1m23.021s for 10978 doc: ~  8 ms per trial (MacBookPro2015)
+# 2019-08-10: real 3s    for 446 documents: ~  7 ms per trial (MacBookPro2015)
 
 cat "$1/euctr-trials-page_"* > "$1/allfiles.txt"
 
@@ -44,13 +44,8 @@ LC_CTYPE=C && LANG=C && < "$1/allfiles.txt" perl -ne '
   next if /^This file contains/;
   next if /A\. Protocol Information/;
   next if /B\. Sponsor Information/;
-  next if /E\. General Information on the Trial/;
   next if /F\. Population of Trial Subjects/;
-  next if /N\. Review by the/;
   next if /P\. End of Trial.$/;
-  next if /G. Investigator Networks/;
-  next if /H.4 Third Country/;
-  next if /MedDRA Classification/;
 
   # remove explanatory information from key F.3.3.1
   next if /^\(For clinical trials recorded/;
@@ -62,6 +57,21 @@ LC_CTYPE=C && LANG=C && < "$1/allfiles.txt" perl -ne '
   # - sponsor records were added but left empty -> create placeholder
   s/^(B\.1\.1 Name of Sponsor:)\s+$/$1 empty/g;
 
+  # - prepare array for meddra
+  s/MedDRA Classification/E.1.2 MedDRA Classification: Yes/g;
+
+  # - prepare array for sponsor supports
+  s/^B.4 Source\(s\) of Monetary or Material Support.*$/B.4 Sources of Monetary or Material Support: Yes/g;
+
+  # - prepare array for networks
+  s/^G. Investigator Networks.*$/G.4 Investigator Networks: Yes/g;
+
+  # - prepare array for inn proposed names
+#  s/^D.3.8 to D.3.10 IMP Identification Details.*$/D.3.8 IMP Identification details: Yes/g;
+
+  # - prepare array for placebos
+  print "\nD.8 Information on Placebo: Yes" if/^D.8 Placebo: 1$/;
+
   # add identifiers for special cases
   s/^(EudraCT Number.*)$/X.1 $1/;
   s/^(Sponsor) ([0-9]+)$/B.1 $1: $2/;
@@ -71,20 +81,13 @@ LC_CTYPE=C && LANG=C && < "$1/allfiles.txt" perl -ne '
   s/^(Link.*)$/X.7 $1/;
 
   # add identifier for end of arrays
-  s/^D\.8 Information on Placebo$/X.9 ENDDMP: TRUE/;
   s/^D\. IMP Identification$/X.9 ENDSPONSOR: TRUE/;
-  # for details = FALSE it is:
-  s/^Full Title:/\nX.9 ENDSPONSOR: TRUE\nxxxxxxxxxxA.3 Full Title of the Trial:/;
 
-  # add identifiers for summary documents (terminating - needed for later step)
-  s/^Sponsor Protocol Number: (.*)$/A.4.1 Sponsors protocol code number: $1/;
-  s/^Sponsor Name: (.*)$/B.1 Sponsor: 1xxxxxxxxxxB.1.1 Name of sponsor: $1/;
-  s/^Full Title: (.*)$/A.3 Full Title of the Trial: $1-/;
-  s/^(Start Date:.*)$/X.7 $1/;
-  s/^Medical condition: (.*)$/E.1.1 Medical conditions being investigated: $1/;
-  s/^Disease: (.*)$/E.1.1.2 Therapeutic area: $1/;
-  s/^(Population Age.*)$/X.8 $1/;
-  s/^(Gender.*)$/X.9 $1/;
+  print "\nX.9 ENDDMP: TRUE"      if /^D\.8 Information on Placebo$/;
+  print "\nX.9 ENDMEDDRA: TRUE"   if /^E\.1\.3 Condition/;
+  print "\nX.9 ENDSUPPORT: TRUE"  if /^B\.5 Contact/;
+  print "\nX.9 ENDNETWORK: TRUE"  if /^N\. Review|^H\.4 Third Country/;
+#  print "\nX.9 ENDIMPIDENT: TRUE" if /^D\.3\.11 The IMP contains an/;
 
   # sanitise file
   s/\t/ /g;
@@ -97,10 +100,7 @@ LC_CTYPE=C && LANG=C && < "$1/allfiles.txt" perl -ne '
 
   # create id per record from eudract number followed by
   # country 2 character id or by "3rd" for non-EU countries
-  # - for details document
   s/^X.7 Link.*search\/trial\/([0-9-]*)\/([A-Z][A-Z]|3rd)\/$/xxxxxxxxxx"_id": "$1-\U$2\E"/;
-  # - for summary document
-  s/^X.7 Link.*search\/.*:([0-9-]{14})$/xxxxxxxxxx"_id": "$1"/;
 
   # crude attack on newlines within variable fields
   s/^([ABDEFGHNPX][.][1-9 I].+)$/\nxxxxxxxxxx$1/g;
@@ -132,6 +132,11 @@ sed \
   -e 's/\("x1_eudract_number.*$\)/}{/g' \
   -e 's/^"dimp": "1",$/"dimp": [ { "_dimp": "1",/g' \
   -e 's/^"b1_sponsor": "1",$/"b1_sponsor": [ { "_b1_sponsor": "1",/g' \
+  -e 's/^"e12_meddra_classification": "Yes",$/"e12_meddra_classification": [/g' \
+  -e 's/^"b4_sources_of_monetary_or_material_support": "Yes",$/"b4_sources_of_monetary_or_material_support": [/g' \
+  -e 's/^"g4_investigator_networks": "Yes",$/"g4_investigator_networks": [/g' \
+  -e 's/^"d38_imp_identification_details": "Yes",$/"d38_imp_identification_details": [{/g' \
+  -e 's/^"d8_information_on_placebo": "Yes",$/"d8_information_on_placebo": [/g' \
   -e '/^["{}]/!d' \
   -e '/""/d' \
   | \
@@ -140,6 +145,7 @@ sed \
   -e '$ s/\(.*\),/\1}/' \
   | \
 perl -pe 'BEGIN{undef $/;}
+
   # delete comma from last line in record
   s/,\n\}\{/\}\nNEWRECORDIDENTIFIER\n\{/g ;
 
@@ -147,13 +153,44 @@ perl -pe 'BEGIN{undef $/;}
   s/("d[0-9]+_.*"),\n"dimp": "([2-9])",/$1\}, \n\{ "_dimp": "$2",/g ;
   s/("d[0-9]+_.*"),\n"x9_enddmp.*/$1\}\n],/g ;
 
-  # create array with sponsor(s), closed before:
-  # dimp or e11_ or a3_full_ elements
-  s/("b[0-9]+_.*"),\n"b1_sponsor": "([2-9])",/$1\}, \n\{ "_b1_sponsor": "$2",/g ;
+  # create array with sponsor(s)
+  s/,\n"b1_sponsor": "([2-9])",/}, \n\{ "_b1_sponsor": "$1",/g ;
   s/("b[0-9]+_.*"),\n"x9_endsponsor.*/$1\}\n],/g ;
 
+  # create array of investigator network from first element
+  s/("g4_investigator_network_to_be_involved_in_the_trial": ".*?"),/},{$1,/g ;
+  s/"x9_endnetwork": "TRUE"/} ]/g ;
+
+  # create array of meddra terms
+  s/("e12_version": ".*?"),/},{$1,/g ;
+  s/"x9_endmeddra": "TRUE"/} ]/g ;
+
+  # create array of b4_source_of_monetary_or_material_support terms
+  s/("b41_name_of_organisation_providing_support": ".*?"),/},{$1,/g ;
+  s/"x9_endsupport": "TRUE"/} ]/g ;
+  s/"x9_endsponsor": "TRUE"/} ]/g ;
+
+  # create array of imp identification details
+  s/("d38_inn__proposed_inn": ".*?"|"d393_other_descriptive_name": ".*?"),/},{$1,/g ;
+  # details may be missing in some records,
+#  s/"d3[189][^,]+": "[^\"]+",\n"x9_endimpident": "TRUE"/} ] /g ;
+  # thus delete end identifier if there was no array
+#  s/"x9_endimpident": "TRUE",//g ;
+
+  # create array of placebos
+  s/("d8_placebo": ".*?"),/},{$1,/g ;
+  s/(d8.*\n)("e11_)/$1} ], $2/g ;
+
+  # correct formatting artefacts
+  s/{\n?},//g ;
+  s/\[\n?},/[/g ;
+  s/,\n?}/}/g ;
+  # empty array
+  s/\[\n?} ?\]/[]/g ;
+
   ' | \
 perl -pe '
+
   # write NDJSON
   s/\n//g ;
   s/NEWRECORDIDENTIFIER/\n/g ;
